<?php
/**
 * This is the PHPWS_Section class. It holds data for a single section in a page.
 * It also contains functions for manipulation of the data and updating or
 * deleting this section.
 *
 * @version $Id: Section.php,v 1.27 2003/05/30 15:45:30 adam Exp $
 * @author  Adam Morton <adam@NOSPAM.tux.appstate.edu>
 * @package PageMaster
 */
class PHPWS_Section {
  /**
   * Database id of this section
   * @var    integer
   * @access private
   */
  var $id = NULL;

  /**
   * Title of this section
   * @var    string
   * @access private
   */
  var $title = NULL;

  /**
   * Textual body of this section
   * @var    string
   * @access private
   */
  var $text = NULL;

  /**
   * An array of image attributes for this section's image
   * @var    array
   * @access private
   */
  var $image = NULL;

  var $template = NULL;

  /**
   * Constructor for the PHPWS_Section object.
   *
   * @param  integer $SECT_id Database id of the section's data to be loaded into the class.
   * @access public
   */
  function PHPWS_Section ($SECT_id = NULL) {
    if($SECT_id) {
      $result = $GLOBALS["core"]->sqlSelect("mod_pagemaster_sections", "id", $SECT_id);

      $this->id = $SECT_id;
      $this->title = $result[0]["title"];
      $this->text = $result[0]["text"];
      if (!empty($result[0]["image"]))
	$this->image = unserialize($result[0]["image"]);
      $this->template = $result[0]["template"];
    } else {
      $this->image = array();
      $this->template = "default.tpl";
    }
  }// END FUNC PHPWS_Section()

  /**
   * Displays this section, usually in the context of a page object.
   *
   * @param  boolean $edit_mode Simple flag to tell section whether or not the page is in edit mode.
   * @access public
   */
  function view_section($edit_mode=FALSE) {
    include($GLOBALS["core"]->source_dir . "mod/pagemaster/conf/config.php");
    $content = NULL;
    if (isset($this->image['alt']))
      $altTag = $this->image['alt'];
    else
      $altTag = NULL;

    if($this->template) {
      if(isset($this->image["name"])) {
	$image_string = "<img src=\"$image_directory" . $this->image["name"] .
	  "\" width=\"" . $this->image["width"] . "\" height=\"" . $this->image["height"] .
	  "\" alt=\"" . $altTag . "\" />";
      }

      $template_array['CREATED_INFO'] =
	 $_SESSION["translate"]->it("Created on [var1] by [var2]",
				    $_SESSION["SES_PM_page"]->created_date,
				    $_SESSION["SES_PM_page"]->created_username);
      $template_array['UPDATED_INFO'] =
	 $_SESSION["translate"]->it("Updated on [var1] by [var2]",
				    $_SESSION["SES_PM_page"]->updated_date,
				    $_SESSION["SES_PM_page"]->updated_username);
      $template_array["CREATED_DATE"] = $_SESSION["SES_PM_page"]->created_date;
      $template_array["UPDATED_DATE"] = $_SESSION["SES_PM_page"]->updated_date;
      $template_array["CREATED_USER"] = $_SESSION["SES_PM_page"]->created_username;
      $template_array["UPDATED_USER"] = $_SESSION["SES_PM_page"]->updated_username;
      $template_array["IMAGE"] = (isset($image_string)) ? $image_string : NULL;
      $template_array["TITLE"] = $GLOBALS["core"]->parseOutput($this->title);
      $template_array["TEXT"] = $GLOBALS["core"]->parseOutput($this->text);

      $useTemplate = $this->template;
      $themeDir = $_SESSION['OBJ_layout']->theme_dir . "templates/pagemaster/" . $useTemplate;
      $templateDir = PHPWS_SOURCE_DIR . "mod/pagemaster/templates/$useTemplate";

      if (!is_file($themeDir) && !is_file($templateDir)){
	$useTemplate = "default.tpl";
	if ($_SESSION['OBJ_user']->allow_access("pagemaster")){
	  $error = new PHPWS_Error("pagemaster", "view_section", "<b>" . $_SESSION["translate"]->it("Warning") ."</b>"
				   . ": " . $_SESSION["translate"]->it("Unable to find requested template")
				   . " <b>" . $this->template . "</b>");
	  $error->message("CNT_pagemaster");
	  
	}
      }

      if(isset($_GET["PAGE_user_op"]) && $_GET["PAGE_user_op"] == "view_printable") {
	echo $GLOBALS["core"]->processTemplate($template_array, "pagemaster", $useTemplate);
      } else {
	$content .=
	  $GLOBALS["core"]->processTemplate($template_array, "pagemaster", $useTemplate);
      }
    }

    if($edit_mode) {
      if($_SESSION["OBJ_user"]->js_on) {
	$js_vars["message"] = $_SESSION["translate"]->it("Are you sure you want to delete") .
	  " " . strip_tags($this->title) . "?";
	$js_vars["name"] = $_SESSION["translate"]->it("Remove");
	$js_vars["value"] = "remove";
	$js_vars["location"] = "index.php?module=pagemaster&PAGE_op=remove&SECT_id=" . $this->id;

	$remove_button =
	  $GLOBALS["core"]->js_insert("confirm", "SECT_edit_delete_" . $this->id, $this->id, 0, $js_vars);
      } else {
	$remove_button =
	  $GLOBALS["core"]->formSubmit($_SESSION["translate"]->it("Remove"), "PAGE_op[remove]");
      }

      $myelements[0] = $GLOBALS["core"]->formHidden("module", "pagemaster");
      $myelements[0] .= $GLOBALS["core"]->formHidden("SECT_id", $this->id);
      $myelements[0] .= $remove_button . $_SESSION["OBJ_help"]->show_link("pagemaster", "section_remove");

      $myelements[0] .= "<br />" .
	$GLOBALS["core"]->formSubmit($_SESSION["translate"]->it("Edit"), "SECT_op[edit_section]") .
	$_SESSION["OBJ_help"]->show_link("pagemaster", "section_edit") . "&nbsp;";

      $content .= $GLOBALS["core"]->makeForm("SECT_edit_delete_" . $this->id, "index.php", $myelements, "post", 0, 0) . "<hr />";
    }
    return $content;
  }// END FUNC view_section()

  /**
   * Displays an editing interface for this section.
   *
   * @access public
   */
  function edit_section() {
    include(PHPWS_SOURCE_DIR . "mod/pagemaster/conf/config.php");

    if($this->id) {
      $section_title = "<div class=\"bg_dark\"><b>" . $_SESSION["translate"]->it("Edit Section") . "</b></div>";
    } else {
      $section_title = "<div class=\"bg_dark\"><b>" . $_SESSION["translate"]->it("New Section") . "</b></div>";
    }

    $myelements[0] = $GLOBALS["core"]->formHidden("module", "pagemaster");

    $myelements[0] .= $_SESSION["translate"]->it("Subtitle") . ":<br />" .
      $GLOBALS["core"]->js_insert("wysiwyg", "SECT_edit", "SECT_title");

    $myelements[0] .=
      $GLOBALS["core"]->formTextField("SECT_title", htmlspecialchars($this->title), 33, 1000) .
      $_SESSION["OBJ_help"]->show_link("pagemaster", "section_title") . "<br />";

    $myelements[0] .= $_SESSION["translate"]->it("Text") . ":<br />" .
      $GLOBALS["core"]->js_insert("wysiwyg", "SECT_edit", "SECT_text");

    $myelements[0] .= $GLOBALS["core"]->formTextArea("SECT_text", $this->text, 10, 60) .
      $_SESSION["OBJ_help"]->show_link("pagemaster", "section_text") . "<br />";

    if(isset($this->image["name"])) {
      $imageName = $this->image["name"];
      $myelements[0] .= "<img src=\"" .$image_directory . $this->image["name"] . "\" /><br />" .
	$GLOBALS["core"]->formCheckBox("removeImage", 1, NULL, NULL, $_SESSION["translate"]->it("Remove Image?")) . "<br />";
    } else
      $imageName = NULL;

    if(!isset($this->image["alt"])) {
      $this->image["alt"] = NULL;
    }

    $myelements[0] .=
      $GLOBALS["core"]->formFile("SECT_image", 33, 255, $_SESSION["translate"]->it("Image") . ":<br />") .
      $_SESSION["OBJ_help"]->show_link("pagemaster", "section_image") . "<br />";

    $myelements[0] .= $GLOBALS["core"]->formTextField("SECT_alt", $this->image["alt"], 33, 255,
      $_SESSION["translate"]->it("Short Image Description") . ":<br />") .
      $_SESSION["OBJ_help"]->show_link("pagemaster", "section_image_desc") . "<br />";

    if(!($temp_dir=@opendir("themes/".$_SESSION["OBJ_layout"]->current_theme."/templates/pagemaster/"))) {
      $temp_dir = opendir(PHPWS_SOURCE_DIR . $template_directory);
    }

    while($current_template = readdir($temp_dir)) {
      if (preg_match("/\.tpl\$/i", $current_template))
	$opt_array[$current_template] = $current_template;
    }

    $myelements[0] .= $GLOBALS["core"]->formSelect("SECT_template", $opt_array, $this->template,
      NULL, NULL, NULL, $_SESSION["translate"]->it("Template") . ":<br />") .
      $_SESSION["OBJ_help"]->show_link("pagemaster", "section_template") . "<br /><br />";

    $myelements[0] .= $GLOBALS["core"]->formSubmit($_SESSION["translate"]->it("Save Section"),
      "SECT_op[save_section]") . $_SESSION["OBJ_help"]->show_link("pagemaster", "section_save");

    $GLOBALS["CNT_pagemaster"]["content"] .= $section_title .
      $GLOBALS["core"]->makeForm("SECT_edit", "index.php", $myelements, "post", 0, 1) .
      "<br /><div class=\"bg_dark\">&nbsp;</div>";
  }// END FUNC edit_section

  /**
   * Saves changes to this section in the database.
   *
   * @access public
   */
  function save_section () {
    include(PHPWS_SOURCE_DIR . "mod/pagemaster/conf/config.php");

    $_SESSION["SES_PM_error"] = NULL;

    if(!$_POST["SECT_title"] && !$_POST["SECT_text"] && !$_FILES["SECT_image"]["name"] &&
       !$_POST["SECT_alt"]) {
      $_SESSION["SES_PM_error"] = "empty_section";
      return;
    }

    if(isset($_POST["removeImage"])) {
      if($this->image["name"] && @unlink(PHPWS_HOME_DIR . $image_directory . $this->image["name"])) {
	$this->image = array();
      } else {
	$_SESSION["SES_PM_error"] = "remove_image";
      }
    }

    if(!empty($_FILES["SECT_image"]["name"])) {
      $image = EZform::saveImage("SECT_image", $image_directory, 1024, 1000);
      if(is_array($image)) {
	$this->image = $image;
      } else {
	$_SESSION["SES_PM_error"] = $image;
      }
    }

    if(!empty($_FILES["SECT_image"]["name"]) || isset($this->image["name"])) {
      if($_POST["SECT_alt"]) {
	$this->image["alt"] = $_POST["SECT_alt"];
      } else {
	$_SESSION["SES_PM_error"] = "alt_tag";
      }
    }

    $this->title = PHPWS_Core::parseInput($_POST["SECT_title"]);
    $this->text = PHPWS_Core::parseInput($_POST["SECT_text"]);
    $this->template = $_POST["SECT_template"];

    /* Setup query data for this section. */
    $query_data["title"] = $this->title;    $query_data["text"] = $this->text;
    $query_data["image"] = serialize($this->image);
    $query_data["template"] = $this->template;

    /* Setup the query data for the current page.  Update info. */
    $query_data2["updated_date"] = date("Y-m-d H:i:s");
    $query_data2["updated_username"] = $_SESSION["OBJ_user"]->username;

    if($this->id) {
      $GLOBALS["core"]->sqlUpdate($query_data, "mod_pagemaster_sections", "id", $this->id);
      $this = new PHPWS_Section($this->id);
      $_SESSION["SES_PM_page"]->update_section($this->id);
    } else {
      $this->id = $GLOBALS["core"]->sqlInsert($query_data, "mod_pagemaster_sections", FALSE, TRUE);
      $this = new PHPWS_Section($this->id);
      $_SESSION["SES_PM_page"]->add_section();
    }

    $GLOBALS["core"]->sqlUpdate($query_data2, "mod_pagemaster_pages", "id", $_SESSION["SES_PM_page"]->id);
  }// END FUNC save_section()

  /**
   * Deletes this section from the database.
   *
   * @access public
   */
  function delete_section() {
    $GLOBALS["core"]->sqlDelete("mod_pagemaster_sections", "id", $this->id);
  }// END FUNC delete_section()

  /**
   * Prints an error cooresponding to this section.  The error message is selected
   * by passing the function an error $type
   *
   * @param  string $type Type of error that occured.
   * @access public
   */
  function error() {
    $title = "<div class=\"errortext\">" . $_SESSION["translate"]->it("ERROR!") . "</div>";

    if(PHPWS_Error::isError($_SESSION["SES_PM_error"])) {
      $_SESSION["SES_PM_error"]->message("CNT_pagemaster");
    } else {
      switch($_SESSION["SES_PM_error"]) {
        case "image_upload":
	$content = $_SESSION["translate"]->it("There was a problem uploading the image you submitted!<br />Please check the permissions on your image directory and try again.");
	break;

        case "alt_tag":
	$content = $_SESSION["translate"]->it("You must provide a short description for your image.");
	break;

        case "remove_image":
	  $content = $_SESSION["translate"]->it("There was a problem removing the image from the file system.");
        break;

        case "alt_with_no_image":
	$content = $_SESSION["translate"]->it("You have entered a short image description but no image is associated with this section.");
        break;

        case "empty_section":
	$content = $_SESSION["translate"]->it("You have attempted to save an empty section.");
	break;
      }
      $_SESSION["OBJ_layout"]->popbox($title, $content, NULL, "CNT_pagemaster") . "<br />";
    }

    $_SESSION["SES_PM_error"] = NULL;
  }// END FUNC error()

}// END CLASS PHPWS_Section

?>
