#!/bin/sh

#
# Author: Jeremy Agee <jagee@tux.appstate.edu>
# Version: 0.1
#

# make sure your APACHE_USER and PHPWS_GROUP is set appropriately
# usually nobody or apache for the web server

# SEE: docs/SECURE.txt for details

APACHE_USER=apache
APACHE_GROUP=apache

if [ $# -ne 1 ]; then
    echo ""
    echo Usage:
    echo $0 setup
    echo or
    echo $0 run
    echo ""
    echo Use \'setup\' before running the phpwebsite setup.
    echo Use \'run\' for normal phpwebsite operation.
    echo ""
    exit 127
fi

if [ "$1" == "setup" ]; then

    chown $APACHE_USER:$APACHE_GROUP ../conf/
    chown -R $APACHE_USER:$APACHE_GROUP ../images/
    chown -R $APACHE_USER:$APACHE_GROUP ../files/

    find ../ -type d | xargs chmod 775
    find ../ -type f | xargs chmod 664
    chmod 755 *.sh

    echo Permissions set!  Ready to run phpwebsite setup!

elif [ "$1" == "run" ]; then
    if [ $# -ne 2 ]; then
      echo 1>&2 You need a username and group
      echo 1>&2 "    "$0 run your_username.your_group
      exit 127
    fi
    if [ -f ../.htaccess ]; then
      chown $APACHE_USER:$APACHE_GROUP ../.htaccess
    else
      touch ../.htaccess
      chown $APACHE_USER:$APACHE_GROUP ../.htaccess
    fi

    chown -R $2 ../*
    chown -R $APACHE_USER:$APACHE_GROUP ../images/
    chown -R $APACHE_USER:$APACHE_GROUP ../files/
    chown $APACHE_USER:$APACHE_GROUP ../conf/branch/

    find ../ -type d | xargs chmod 775
    find ../ -type f | xargs chmod 664
    chmod 755 *.sh

    echo Permissions Set!  Ready for normal operation!
fi

exit 0
